<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator;

use Flat101\EnhancedEcommerce\Model\Config\Source\VariantUse;
use Flat101\EnhancedEcommerce\Model\DataLayer\DataProvider\AdapterDataProvider;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4;
use Flat101\EnhancedEcommerce\Model\Handler;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\Session;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Magento\Sales\Model\Order\Item as OrderItem;
use Psr\Log\LoggerInterface;

class Product
{
    use UnsetProperty;

    /** @var Handler\ProductHandler */
    private $productHandler;

    /** @var Handler\ItemHandler */
    private $itemHandler;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var AdapterDataProvider */
    private $adapterDataProvider;

    /** @var Config */
    private $config;

    /** @var Session */
    private $session;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        Handler\ProductHandler $productHandler,
        Handler\ItemHandler $itemHandler,
        CategoryRepositoryInterface $categoryRepository,
        AdapterDataProvider $adapterDataProvider,
        Config $config,
        Session $session,
        LoggerInterface $logger
    ) {
        $this->productHandler = $productHandler;
        $this->categoryRepository = $categoryRepository;
        $this->itemHandler = $itemHandler;
        $this->adapterDataProvider = $adapterDataProvider;
        $this->config = $config;
        $this->session = $session;
        $this->logger = $logger;
    }

    /**
     * @param ProductEntity|null $product
     * @param null $quantity
     * @param null|OrderItem|QuoteItem $item
     * @param null $additionalInfo
     * @return null | Dto\Product
     */
    public function generate(?ProductEntity $product, $quantity = null, $item = null, $additionalInfo = null): ?DataLayerGa4\Dto\Product
    {
        if (is_null($product)) {
            return null;
        }

        $category = $product->getCategory();
        if (null === $category) {
            $categoryIds = $product->getCategoryIds();

            try {
                $category = $this->categoryRepository->get(\reset($categoryIds));
            } catch (NoSuchEntityException $exception) {
                $this->logger->notice($exception->getMessage());
            }
        }

        if (null !== $category) {
            $this->productHandler->setCategory($category);
        }

        $productDto = new DataLayerGa4\Dto\Product();
        $productDto->item_id = $product->getData($this->productHandler->productIdentifier());
        $productDto->item_name = $product->getName();
        $productDto->price = $this->getPrice($product);

        // TODO: Check if correct category placed here
        $productDto->item_category = $this->productHandler->getCategoryName();

        $categoryPath = $this->productHandler->getCategoryPath(false, true);
        if (is_array($categoryPath)) {
	        $i = 1;
            foreach ($categoryPath as $categoryItem) {
                if ($i == 1) {
                    $productDto->item_category = $categoryItem;
                }
                if ($i > 1) {
                    $productDto->{'item_category' . $i} = $categoryItem;
                }
                $i++;
            }
        } else {
            if (!is_null($categoryPath) && $categoryPath !== '') {
                $productDto->item_category = $categoryPath;
            }
        }

        $productDto->item_position = ($additionalInfo['position']) ?? 0;
        $productDto->item_brand = $this->productHandler->getBrandValue() ?? '';
        $productDto->item_variant = $this->checkVariantValue($item) ?? '';

        if (null !== $quantity) {
            $productDto->quantity = intval($quantity);
        }

        // For default use => we need Quote/Order Item entity
        // For discount use => we use Product entity
        if ($this->config->getVariantUse() === VariantUse::DISCOUNT_USAGE) {
            $item = $product;
        }
        if (null === $item) {
            $this->unset($productDto, ['quantity']);
        }
        if ((0 === $productDto->item_position) || ((is_null($productDto->item_position)))) {
            $this->unset($productDto, ['item_position']);
        }
        if (!isset($productDto->item_category)) {
            $this->unset($productDto, ['item_category']);
        }
        if (!isset($productDto->item_category2)) {
            $this->unset($productDto, ['item_category2']);
        }
        if (!isset($productDto->item_category3)) {
            $this->unset($productDto, ['item_category3']);
        }
        if (!isset($productDto->item_category4)) {
            $this->unset($productDto, ['item_category4']);
        }
        if (!isset($productDto->item_category5)) {
            $this->unset($productDto, ['item_category5']);
        }

        $this->adapterDataProvider->handle($productDto, [
            'category' => $category,
            'product' => $product,
        ]);

        return $productDto;
    }

    private function getPrice(ProductEntity $product): float
    {
        return floatval($product->getSpecialPrice()) ?: floatval($product->getFinalPrice());
    }

    private function checkVariantValue($item): ?string
    {
        return $this->productHandler->checkVariantValue($item);
    }
}
